import 'package:flutter/material.dart';

extension IDKitWidgetPadding on Widget {
  /// Add margins to widgets.
  Widget insets(EdgeInsetsGeometry edgeInsets) {
    return Padding(padding: edgeInsets, child: this);
  }

  Widget insetsAll(double value) {
    return Padding(padding: EdgeInsets.all(value), child: this);
  }

  /// Add vertical or horizontal margins to widgets.
  Widget insetsSymmetric({
    double vertical = 0.0,
    double horizontal = 0.0,
  }) {
    return Padding(
      padding: EdgeInsets.symmetric(
        vertical: vertical,
        horizontal: horizontal,
      ),
      child: this,
    );
  }

  /// Add a margin to one side of the widget.
  Widget insetsOnly({
    double top = 0.0,
    double left = 0.0,
    double right = 0.0,
    double bottom = 0.0,
  }) {
    return Padding(
      padding: EdgeInsets.only(
        top: top,
        left: left,
        right: right,
        bottom: bottom,
      ),
      child: this,
    );
  }

  /// Add margin to Sliver type widget.
  Widget silverInsets(EdgeInsetsGeometry edgeInsets) {
    final String className = toStringShort();
    final isSliverWidget = className.startsWith('Sliver');
    assert(isSliverWidget, 'Please use widgets of type Sliver...');
    return SliverPadding(
      padding: edgeInsets,
      sliver: this,
    );
  }

  Widget silverInsetsAll(double value) {
    final String className = toStringShort();
    final isSliverWidget = className.startsWith('Sliver');
    assert(isSliverWidget, 'Please use widgets of type Sliver...');
    return SliverPadding(
      padding: EdgeInsets.all(value),
      sliver: this,
    );
  }

  /// Add horizontal or vertical margins to Sliver widgets.
  Widget silverInsetsSymmetric(
      {double vertical = 0.0, double horizontal = 0.0}) {
    final String className = toStringShort();
    final isSliverWidget = className.startsWith('Sliver');
    assert(isSliverWidget, 'Please use widgets of type Sliver...');
    return SliverPadding(
      padding: EdgeInsets.symmetric(
        vertical: vertical,
        horizontal: horizontal,
      ),
      sliver: this,
    );
  }

  /// Add a margin to a side of a Sliver widget.
  Widget silverInsetsOnly({
    double top = 0.0,
    double left = 0.0,
    double right = 0.0,
    double bottom = 0.0,
  }) {
    final String className = toStringShort();
    final isSliverWidget = className.startsWith('Sliver');
    assert(isSliverWidget, 'Please use widgets of type Sliver...');
    return SliverPadding(
      padding: EdgeInsets.only(
        top: top,
        left: left,
        right: right,
        bottom: bottom,
      ),
      sliver: this,
    );
  }
}
