import 'package:flutter/material.dart';
import 'package:idkit/src/function/color/func_color.dart';

/// Extension of text style.
extension IDKitTextStyle on TextStyle {
  /// Font weight.
  /// Thin, the least thick
  TextStyle get thin => copyWith(fontWeight: FontWeight.w100);

  /// Extra-light
  TextStyle get extraLight => copyWith(fontWeight: FontWeight.w200);

  /// Light
  TextStyle get light => copyWith(fontWeight: FontWeight.w300);

  /// Normal / regular / plain
  TextStyle get normal => copyWith(fontWeight: FontWeight.w400);
  TextStyle get plain => normal;
  TextStyle get regular => normal;

  /// Medium
  TextStyle get medium => copyWith(fontWeight: FontWeight.w500);

  /// Semi-bold
  TextStyle get semibBold => copyWith(fontWeight: FontWeight.w600);

  /// Bold
  TextStyle get bold => copyWith(fontWeight: FontWeight.w700);

  /// Extra-bold
  TextStyle get extraBold => copyWith(fontWeight: FontWeight.w800);

  /// Black, the most thick
  TextStyle get black => copyWith(fontWeight: FontWeight.w900);

  /// Font style.
  TextStyle get italic => copyWith(fontStyle: FontStyle.italic);

  /// Font color.
  /// [color] : Colors.red
  TextStyle c(Color? color) => copyWith(color: color);

  /// [value] : #151515
  TextStyle hexColor(String? value) => copyWith(color: value?.color);

  /// [value] : #FF151515
  TextStyle aHexColor(String? value) => copyWith(color: value?.aHexColor);

  /// [value] : #151515FF
  TextStyle hexAColor(String? value) => copyWith(color: value?.hexAColor);

  /// Set text size.
  TextStyle fSize(double? value) => copyWith(fontSize: value);

  /// Set text line height.
  TextStyle h(double? value) => copyWith(height: value);

  /// Set text word spacing.
  TextStyle wSpacing(double? value) => copyWith(wordSpacing: value);

  /// Set text letter spacing.
  TextStyle lSpacing(double? value) => copyWith(letterSpacing: value);

  /// Sets the opacity of the color of the text style
  TextStyle oColor(double value) => copyWith(color: color?.withOpacity(value));
}
