import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:idkit/src/widget/choice/widget_choice_func.dart';
import 'package:idkit/src/widget/choice/widget_choice_init.dart';

/// Widget to select function.
class IDKitChoice<T extends Object> extends ChoiceWidget<T> {
  const IDKitChoice.list({
    Key? key,
    required List<T> sources,
    required ItemBuilder<T> itemBuilder,
    Function(T result, List<T> results)? choiceCallMethod,
    int? maximumChoice,
    bool isCancelAll = true,
    bool isStartChoicedCall = false,
    List<int>? chosenIndexs,
    ChoiceType type = ChoiceType.single,
    Axis scrollDirection = Axis.vertical,
    bool reverse = false,
    ScrollController? controller,
    bool? primary,
    ScrollPhysics? physics,
    bool shrinkWrap = false,
    EdgeInsetsGeometry? padding,
    double? itemExtent,
    Widget? prototypeItem,
    bool addAutomaticKeepAlives = true,
    bool addRepaintBoundaries = true,
    bool addSemanticIndexes = true,
    double? cacheExtent,
    int? Function(Key)? findChildIndexCallback,
    int? semanticChildCount,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
    ScrollViewKeyboardDismissBehavior keyboardDismissBehavior =
        ScrollViewKeyboardDismissBehavior.manual,
    String? restorationId,
    Clip clipBehavior = Clip.hardEdge,
  }) : super(
          key: key,
          sort: ChoiceSort.list,
          type: type,
          isCancelAll: isCancelAll,
          isStartChoicedCall: isStartChoicedCall,
          chosenIndexs: chosenIndexs,
          sources: sources,
          itemBuilder: itemBuilder,
          maximumChoice: maximumChoice,
          choiceCallMethod: choiceCallMethod,
          scrollDirection: scrollDirection,
          reverse: reverse,
          controller: controller,
          primary: primary,
          physics: physics,
          shrinkWrap: shrinkWrap,
          padding: padding,
          itemExtent: itemExtent,
          prototypeItem: prototypeItem,
          addAutomaticKeepAlives: addAutomaticKeepAlives,
          addRepaintBoundaries: addRepaintBoundaries,
          addSemanticIndexes: addSemanticIndexes,
          cacheExtent: cacheExtent,
          findChildIndexCallback: findChildIndexCallback,
          semanticChildCount: semanticChildCount,
          dragStartBehavior: dragStartBehavior,
          keyboardDismissBehavior: keyboardDismissBehavior,
          restorationId: restorationId,
          clipBehavior: clipBehavior,
        );

  /// Separated type list choice widget.
  const IDKitChoice.listSeparated({
    Key? key,
    required List<T> sources,
    required ItemBuilder<T> itemBuilder,
    required ItemBuilder<T> separatorBuilder,
    Function(T result, List<T> results)? choiceCallMethod,
    int? maximumChoice,
    bool isCancelAll = true,
    bool isStartChoicedCall = false,
    List<int>? chosenIndexs,
    ChoiceType type = ChoiceType.single,
    Axis scrollDirection = Axis.vertical,
    bool reverse = false,
    ScrollController? controller,
    bool? primary,
    ScrollPhysics? physics,
    bool shrinkWrap = false,
    EdgeInsetsGeometry? padding,
    bool addAutomaticKeepAlives = true,
    bool addRepaintBoundaries = true,
    bool addSemanticIndexes = true,
    double? cacheExtent,
    int? Function(Key)? findChildIndexCallback,
    int? semanticChildCount,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
    ScrollViewKeyboardDismissBehavior keyboardDismissBehavior =
        ScrollViewKeyboardDismissBehavior.manual,
    String? restorationId,
    Clip clipBehavior = Clip.hardEdge,
  }) : super(
          key: key,
          sort: ChoiceSort.separated,
          type: type,
          isCancelAll: isCancelAll,
          isStartChoicedCall: isStartChoicedCall,
          chosenIndexs: chosenIndexs,
          sources: sources,
          itemBuilder: itemBuilder,
          separatorBuilder: separatorBuilder,
          maximumChoice: maximumChoice,
          choiceCallMethod: choiceCallMethod,
          scrollDirection: scrollDirection,
          reverse: reverse,
          controller: controller,
          primary: primary,
          physics: physics,
          shrinkWrap: shrinkWrap,
          padding: padding,
          addAutomaticKeepAlives: addAutomaticKeepAlives,
          addRepaintBoundaries: addRepaintBoundaries,
          addSemanticIndexes: addSemanticIndexes,
          cacheExtent: cacheExtent,
          findChildIndexCallback: findChildIndexCallback,
          semanticChildCount: semanticChildCount,
          dragStartBehavior: dragStartBehavior,
          keyboardDismissBehavior: keyboardDismissBehavior,
          restorationId: restorationId,
          clipBehavior: clipBehavior,
        );

  /// Selection widget for grid type.
  const IDKitChoice.grid({
    Key? key,
    required List<T> sources,
    required ItemBuilder<T> itemBuilder,
    SliverGridDelegate? gridDelegate,
    Function(T result, List<T> results)? choiceCallMethod,
    int? maximumChoice,
    bool isCancelAll = true,
    bool isStartChoicedCall = false,
    List<int>? chosenIndexs,
    ChoiceType type = ChoiceType.single,
    Axis scrollDirection = Axis.vertical,
    bool reverse = false,
    ScrollController? controller,
    bool? primary,
    ScrollPhysics? physics,
    bool shrinkWrap = false,
    EdgeInsetsGeometry? padding,
    double? itemExtent,
    Widget? prototypeItem,
    bool addAutomaticKeepAlives = true,
    bool addRepaintBoundaries = true,
    bool addSemanticIndexes = true,
    double? cacheExtent,
    int? Function(Key)? findChildIndexCallback,
    int? semanticChildCount,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
    ScrollViewKeyboardDismissBehavior keyboardDismissBehavior =
        ScrollViewKeyboardDismissBehavior.manual,
    String? restorationId,
    Clip clipBehavior = Clip.hardEdge,
  }) : super(
          key: key,
          sort: ChoiceSort.grid,
          type: type,
          isCancelAll: isCancelAll,
          isStartChoicedCall: isStartChoicedCall,
          chosenIndexs: chosenIndexs,
          sources: sources,
          itemBuilder: itemBuilder,
          gridDelegate: gridDelegate,
          maximumChoice: maximumChoice,
          choiceCallMethod: choiceCallMethod,
          scrollDirection: scrollDirection,
          reverse: reverse,
          controller: controller,
          primary: primary,
          physics: physics,
          shrinkWrap: shrinkWrap,
          padding: padding,
          itemExtent: itemExtent,
          prototypeItem: prototypeItem,
          addAutomaticKeepAlives: addAutomaticKeepAlives,
          addRepaintBoundaries: addRepaintBoundaries,
          addSemanticIndexes: addSemanticIndexes,
          cacheExtent: cacheExtent,
          findChildIndexCallback: findChildIndexCallback,
          semanticChildCount: semanticChildCount,
          dragStartBehavior: dragStartBehavior,
          keyboardDismissBehavior: keyboardDismissBehavior,
          restorationId: restorationId,
          clipBehavior: clipBehavior,
        );

  /// Selection widget for warp type.
  const IDKitChoice.warp({
    Key? key,
    required List<T> sources,
    ChoiceType type = ChoiceType.single,
    required ItemBuilder<T> itemBuilder,
    Function(T result, List<T> results)? choiceCallMethod,
    int? maximumChoice,
    bool isCancelAll = true,
    bool isStartChoicedCall = false,
    List<int>? chosenIndexs,
    Axis direction = Axis.horizontal,
    WrapAlignment alignment = WrapAlignment.start,
    double spacing = 0.0,
    WrapAlignment runAlignment = WrapAlignment.start,
    double runSpacing = 0.0,
    WrapCrossAlignment crossAxisAlignment = WrapCrossAlignment.start,
    TextDirection? textDirection,
    VerticalDirection verticalDirection = VerticalDirection.down,
  }) : super(
          key: key,
          sort: ChoiceSort.warp,
          type: type,
          isCancelAll: isCancelAll,
          isStartChoicedCall: isStartChoicedCall,
          chosenIndexs: chosenIndexs,
          sources: sources,
          itemBuilder: itemBuilder,
          maximumChoice: maximumChoice,
          choiceCallMethod: choiceCallMethod,
          direction: direction,
          alignment: alignment,
          spacing: spacing,
          runAlignment: runAlignment,
          runSpacing: runSpacing,
          crossAxisAlignment: crossAxisAlignment,
          textDirection: textDirection,
          verticalDirection: verticalDirection,
        );
}
