import 'package:flutter/material.dart';
import 'package:flutter/gestures.dart';
import 'package:idkit/src/widget/choice/widget_choice_func.dart';

class ChoiceWidget<T extends Object> extends StatefulWidget {
  const ChoiceWidget({
    Key? key,
    this.type = ChoiceType.single,
    this.isCancelAll = true,
    this.chosenIndexs,
    this.isStartChoicedCall = false,
    // list
    this.scrollDirection = Axis.vertical,
    required this.sources,
    required this.sort,
    required this.itemBuilder,
    this.separatorBuilder,
    this.choiceCallMethod,
    this.maximumChoice,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.shrinkWrap = false,
    this.padding,
    this.itemExtent,
    this.prototypeItem,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.cacheExtent,
    this.semanticChildCount,
    this.dragStartBehavior = DragStartBehavior.start,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.restorationId,
    this.clipBehavior = Clip.hardEdge,
    this.findChildIndexCallback,
    // grid
    this.gridDelegate,
    // warp
    this.direction = Axis.horizontal,
    this.alignment = WrapAlignment.start,
    this.spacing = 0.0,
    this.runAlignment = WrapAlignment.start,
    this.runSpacing = 0.0,
    this.crossAxisAlignment = WrapCrossAlignment.start,
    this.textDirection,
    this.verticalDirection = VerticalDirection.down,
  }) : super(key: key);

  /// Attribute definition
  final ChoiceType type;
  final ChoiceSort sort;
  final int? maximumChoice;
  final List<T> sources;
  final ItemBuilder<T> itemBuilder;
  final ItemBuilder<T>? separatorBuilder;
  final Function(T result, List<T> results)? choiceCallMethod;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController? controller;
  final bool? primary;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final EdgeInsetsGeometry? padding;
  final double? itemExtent;
  final Widget? prototypeItem;
  final bool addAutomaticKeepAlives;
  final bool addRepaintBoundaries;
  final bool addSemanticIndexes;
  final double? cacheExtent;
  final int? Function(Key)? findChildIndexCallback;
  final int? semanticChildCount;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final String? restorationId;
  // Grid
  final SliverGridDelegate? gridDelegate;

  // warp
  final Axis direction;
  final WrapAlignment alignment;
  final double spacing;
  final WrapAlignment runAlignment;
  final double runSpacing;
  final WrapCrossAlignment crossAxisAlignment;
  final TextDirection? textDirection;
  final VerticalDirection verticalDirection;

  // common
  final Clip clipBehavior;
  final bool isCancelAll;
  final List<int>? chosenIndexs;
  final bool isStartChoicedCall;

  @override
  State<ChoiceWidget<T>> createState() => _ChoiceWidgetState<T>();
}

class _ChoiceWidgetState<T extends Object> extends State<ChoiceWidget<T>> {
  // Back to data processing.
  void handleReturnData() {
    final Iterable<ChoiceState<T>> iterable =
        dataList.where((element) => element.state == true);
    if (iterable.isNotEmpty) {
      widget.choiceCallMethod?.call(
        iterable.first.data,
        iterable.map((e) => e.data).toList(),
      );
    }
  }

  // Handling click events.
  void handleTapEvent(ChoiceState<T> model) {
    if (widget.type == ChoiceType.single) {
      final ChoiceState<T> inquireModel = dataList
          .singleWhere((element) => element.state == true, orElse: () => model);
      if (inquireModel == model && widget.isCancelAll) {
        inquireModel.state = !inquireModel.state;
      } else {
        inquireModel.state = false;
        model.state = true;
      }
    } else {
      final Iterable<ChoiceState<T>> iterable =
          dataList.where((element) => element.state == true);
      final int? count = widget.maximumChoice;
      if (count != null && iterable.isNotEmpty) {
        final int length = iterable.length;
        late bool isEqual = (length == count && model.state == false);
        final bool isMaxCount = length > count || isEqual;
        assert(
          !isMaxCount,
          'The number of multiple selections cannot exceed the maximum limit.',
        );
      }
      if (!widget.isCancelAll &&
          iterable.length == 1 &&
          iterable.first == model) return;
      model.state = !model.state;
    }
  }

  // Data callback processing
  void dataCallBackProcess(ChoiceState<T> data) {
    if (mounted) {
      handleTapEvent(data);
      handleReturnData();
      setState(() {});
    }
  }

  // Data conversion pool
  List<ChoiceState<T>> dataList = [];

  // Process incoming data
  void handleIncomingData() {
    dataList = widget.sources.map((e) => ChoiceState(data: e)).toList();
    if (widget.chosenIndexs != null && widget.chosenIndexs!.isNotEmpty) {
      if (widget.type == ChoiceType.single) {
        final int index = widget.chosenIndexs!.last;
        assert(index <= dataList.length && index >= 0,
            'Set the selected index beyond the selected range!');
        dataList[index].state = true;
      } else {
        for (int index in widget.chosenIndexs!) {
          assert(index <= dataList.length && index >= 0,
              'Set the selected index beyond the selected range!');
          dataList[index].state = true;
        }
      }
    }
  }

  @override
  void initState() {
    handleIncomingData();
    if (widget.isStartChoicedCall) handleReturnData();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    late Widget child = const SizedBox();
    switch (widget.sort) {
      case ChoiceSort.list:
        child = ListView.builder(
          key: widget.key,
          scrollDirection: widget.scrollDirection,
          reverse: widget.reverse,
          controller: widget.controller,
          primary: widget.primary,
          physics: widget.physics,
          shrinkWrap: widget.shrinkWrap,
          padding: widget.padding,
          itemExtent: widget.itemExtent,
          prototypeItem: widget.prototypeItem,
          itemBuilder: (context, index) {
            final ChoiceState<T> data = dataList[index];
            return InkWell(
              child: widget.itemBuilder(context, data.state, data.data),
              onTap: () => dataCallBackProcess(data),
            );
          },
          itemCount: widget.sources.length,
          findChildIndexCallback: widget.findChildIndexCallback,
          addAutomaticKeepAlives: widget.addAutomaticKeepAlives,
          addRepaintBoundaries: widget.addRepaintBoundaries,
          addSemanticIndexes: widget.addSemanticIndexes,
          cacheExtent: widget.cacheExtent,
          semanticChildCount: widget.semanticChildCount,
          dragStartBehavior: widget.dragStartBehavior,
          keyboardDismissBehavior: widget.keyboardDismissBehavior,
          restorationId: widget.restorationId,
          clipBehavior: widget.clipBehavior,
        );
        break;
      case ChoiceSort.separated:
        child = ListView.separated(
          key: widget.key,
          scrollDirection: widget.scrollDirection,
          reverse: widget.reverse,
          controller: widget.controller,
          primary: widget.primary,
          physics: widget.physics,
          shrinkWrap: widget.shrinkWrap,
          padding: widget.padding,
          itemBuilder: (context, index) {
            final ChoiceState<T> data = dataList[index];
            return InkWell(
              child: widget.itemBuilder(context, data.state, data.data),
              onTap: () => dataCallBackProcess(data),
            );
          },
          separatorBuilder: (BuildContext context, int index) {
            final ChoiceState<T> data = dataList[index];
            return widget.separatorBuilder
                    ?.call(context, data.state, data.data) ??
                const Divider();
          },
          itemCount: widget.sources.length,
          findChildIndexCallback: widget.findChildIndexCallback,
          addAutomaticKeepAlives: widget.addAutomaticKeepAlives,
          addRepaintBoundaries: widget.addRepaintBoundaries,
          addSemanticIndexes: widget.addSemanticIndexes,
          cacheExtent: widget.cacheExtent,
          dragStartBehavior: widget.dragStartBehavior,
          keyboardDismissBehavior: widget.keyboardDismissBehavior,
          restorationId: widget.restorationId,
          clipBehavior: widget.clipBehavior,
        );
        break;
      case ChoiceSort.grid:
        child = GridView.builder(
          key: widget.key,
          scrollDirection: widget.scrollDirection,
          reverse: widget.reverse,
          controller: widget.controller,
          primary: widget.primary,
          physics: widget.physics,
          shrinkWrap: widget.shrinkWrap,
          padding: widget.padding,
          gridDelegate: widget.gridDelegate ??
              const SliverGridDelegateWithFixedCrossAxisCount(
                  crossAxisCount: 2),
          itemBuilder: (BuildContext context, int index) {
            final ChoiceState<T> data = dataList[index];
            return InkWell(
              child: widget.itemBuilder(context, data.state, data.data),
              onTap: () => dataCallBackProcess(data),
            );
          },
          findChildIndexCallback: widget.findChildIndexCallback,
          itemCount: widget.sources.length,
          addAutomaticKeepAlives: widget.addAutomaticKeepAlives,
          addRepaintBoundaries: widget.addRepaintBoundaries,
          addSemanticIndexes: widget.addSemanticIndexes,
          cacheExtent: widget.cacheExtent,
          semanticChildCount: widget.semanticChildCount,
          dragStartBehavior: widget.dragStartBehavior,
          keyboardDismissBehavior: widget.keyboardDismissBehavior,
          restorationId: widget.restorationId,
          clipBehavior: widget.clipBehavior,
        );
        break;
      case ChoiceSort.warp:
        child = Wrap(
          key: widget.key,
          direction: widget.direction,
          alignment: widget.alignment,
          spacing: widget.spacing,
          runAlignment: widget.runAlignment,
          runSpacing: widget.runSpacing,
          crossAxisAlignment: widget.crossAxisAlignment,
          textDirection: widget.textDirection,
          verticalDirection: widget.verticalDirection,
          clipBehavior: widget.clipBehavior,
          children: dataList.map((data) {
            return InkWell(
              child: widget.itemBuilder.call(context, data.state, data.data),
              onTap: () => dataCallBackProcess(data),
            );
          }).toList(),
        );
        break;
    }
    return child;
  }
}

/// Choose sort of widget
enum ChoiceSort {
  list,
  separated,
  grid,
  warp,
}
