<p id='jump'></p>

## 快速选择组件

#### 简介
该组件主要是快速实现多项选择或者单项选择模块视图以及多项选择时也支持设置最大选择数量。该组件使用时你只需要负责传入选择的数据和选择返回数据类型以及选中和未选中的视图效果即可。该组件包含的样式如下:

- IDKitChoice.list   列表类型的多选或单选
- IDKitChoice.listSeparated  列表带间隔类型的多选或单选
- IDKitChoice.grid   网格类型的多选或单选
- IDKitChoice.warp   子组件自动根据大小调整位置多选或单选

#### 示例展示
```dart
/// Choice Widget
class ChoiceWidget extends StatelessWidget {
  const ChoiceWidget({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<ChoiceState<String>> list = ['王昭君', '甄姬', '妲己', '不知火舞', '貂蝉']
        .map((e) => ChoiceState(data: e))
        .toList();
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        /// list
        Text(
          'IDKitChoice.list',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.list(
          type: ChoiceType.single,
          shrinkWrap: true,
          sources: list,
          mMaxCount: 2,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 200,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),

        /// listSeparated
        Text(
          'IDKitChoice.listSeparated',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.listSeparated(
          type: ChoiceType.multiple,
          shrinkWrap: true,
          sources: list,
          mMaxCount: 2,
          separatorBuilder: (_, ChoiceState<String> result) {
            return const Divider(
              height: 2,
              color: Colors.red,
              thickness: 2,
            );
          },
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 200,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          multipleValueCall: (list) {
            debugPrint(list.toString());
          },
        ).insetsOnly(bottom: 20),

        /// warp
        Text(
          'IDKitChoice.warp',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.warp(
          type: ChoiceType.single,
          sources: list,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return ColoredBox(
              color: color,
              child: Text(data.data ?? ''),
            ).insetsOnly(right: 10);
          },
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),

        /// grid
        Text(
          'IDKitChoice.grid',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.grid(
          type: ChoiceType.single,
          sources: list,
          shrinkWrap: true,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 20,
              height: 20,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: 3,
            childAspectRatio: 3,
          ),
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),
      ],
    );
  }
}
```

#### 示例展示结果
![choice](https://github.com/zhoushuangjian001/idkit/blob/master/readme_img/choice.png?raw=true)

#### 便捷方式
**[❤️ &nbsp;&nbsp;Pub-Idkit](https://pub.flutter-io.cn/packages/idkit)** &nbsp; &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;**[👈🏻 &nbsp;&nbsp;目录](https://github.com/zhoushuangjian001/idkit/blob/master/README_CH.md)** &nbsp; &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;**[ 👆🏻 &nbsp;&nbsp;顶部](#jump)**