<p id='jump'></p>

## Quick selection of widget

#### Introduction
This widgets is mainly used to quickly implement multiple selection or single selection module view and also supports setting the maximum number of selections when multiple selections are made. When using this component, you only need to be responsible for the incoming selected data and the selected return data type, as well as the selected and unselected view effects. This widget contains the following styles:

- IDKitChoice.list   Multiple selection or single selection of list type
- IDKitChoice.listSeparated  List multiple or single selection with interval type
- IDKitChoice.grid   Multiple or single selection of grid type
- IDKitChoice.warp   Subcomponents automatically adjust the position according to the size of the multi-select or single-select

#### Example Display
```dart
/// Choice Widget
class ChoiceWidget extends StatelessWidget {
  const ChoiceWidget({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<ChoiceState<String>> list = ['王昭君', '甄姬', '妲己', '不知火舞', '貂蝉']
        .map((e) => ChoiceState(data: e))
        .toList();
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        /// list
        Text(
          'IDKitChoice.list',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.list(
          type: ChoiceType.single,
          shrinkWrap: true,
          sources: list,
          mMaxCount: 2,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 200,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),

        /// listSeparated
        Text(
          'IDKitChoice.listSeparated',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.listSeparated(
          type: ChoiceType.multiple,
          shrinkWrap: true,
          sources: list,
          mMaxCount: 2,
          separatorBuilder: (_, ChoiceState<String> result) {
            return const Divider(
              height: 2,
              color: Colors.red,
              thickness: 2,
            );
          },
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 200,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          multipleValueCall: (list) {
            debugPrint(list.toString());
          },
        ).insetsOnly(bottom: 20),

        /// warp
        Text(
          'IDKitChoice.warp',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.warp(
          type: ChoiceType.single,
          sources: list,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return ColoredBox(
              color: color,
              child: Text(data.data ?? ''),
            ).insetsOnly(right: 10);
          },
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),

        /// grid
        Text(
          'IDKitChoice.grid',
          style: const TextStyle().bold.fSize(20),
        ),
        IDKitChoice<String>.grid(
          type: ChoiceType.single,
          sources: list,
          shrinkWrap: true,
          itemBuilder: (context, data) {
            final color = data.state ? Colors.red : Colors.blue;
            return Container(
              width: 20,
              height: 20,
              color: color,
              alignment: Alignment.center,
              child: Text(data.data ?? ''),
            );
          },
          gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: 3,
            childAspectRatio: 3,
          ),
          valueCall: (ChoiceState<String> result) {
            debugPrint(result.data);
          },
        ).insetsOnly(bottom: 20),
      ],
    );
  }
}
```
#### Sample Display Results
![choice](https://github.com/zhoushuangjian001/idkit/blob/master/readme_img/choice.png?raw=true)

#### Convenient Way
**[❤️ &nbsp;&nbsp;Pub-Idkit](https://pub.flutter-io.cn/packages/idkit)** &nbsp; &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;**[👈🏻 &nbsp;&nbsp;Contents](https://github.com/zhoushuangjian001/idkit/blob/master/README.md)** &nbsp; &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;**[ 👆🏻 &nbsp;&nbsp;Top](#jump)**